/*! For license information please see 506.05c9fe0440c840c65f1a.js.LICENSE.txt */
"use strict";(self.webpackChunkneurosift_jp=self.webpackChunkneurosift_jp||[]).push([[125,506],{36506:(e,t,r)=>{r.r(t),r.d(t,{HBoxLayout:()=>o,Hyperlink:()=>s,SmallIconButton:()=>a,VBoxLayout:()=>f,VerticalScrollView:()=>N,isArrayOf:()=>g,isBoolean:()=>b,isEqualTo:()=>k,isFunction:()=>p,isJSONObject:()=>x,isJSONSerializable:()=>S,isJSONValue:()=>O,isNull:()=>y,isNumber:()=>h,isObject:()=>v,isObjectOf:()=>m,isOneOf:()=>w,isString:()=>d,optional:()=>j,useWindowDimensions:()=>u,validateObject:()=>_});var i=r(74848),n=r(93345),l=r.n(n);const o=({widths:e,height:t,spacing:r=0,children:o})=>{const s=e.reduce(((e,t)=>e+t),0),a=l().Children.toArray(o).map((e=>e)),c=(0,n.useMemo)((()=>{const t=[];let i=0;for(const n of e)t.push(i),i+=n+r;return t}),[e,r]);return(0,i.jsx)("div",{className:"HBoxLayout",style:{position:"relative",width:s,height:t},children:a.map(((r,n)=>r?(0,i.jsx)("div",{style:{position:"absolute",overflow:"hidden",background:"white",left:c[n],top:0,width:e[n],height:t},children:(0,i.jsx)(r.type,{...r.props,width:e[n],height:t})},n):(0,i.jsx)("span",{})))})},s=({children:e,onClick:t,color:r,disabled:n,href:l,target:o,title:s})=>l?(0,i.jsx)("a",{href:l,title:s,target:o,style:{cursor:"pointer",color:r||"darkblue"},children:e}):n?(0,i.jsx)("span",{title:s,style:{color:"gray"},children:e}):(0,i.jsx)("a",{onClick:t,title:s,style:{cursor:"pointer",color:r||"darkblue"},children:e}),a=({icon:e,onClick:t,title:r,label:n,disabled:l,fontSize:o})=>{const s=["SmallIconButton"];return l?s.push("disabled"):s.push("enabled"),(0,i.jsxs)("span",{className:s.join(" "),title:r,onClick:l?void 0:t,style:{cursor:l?"default":"pointer",fontSize:.8*(o||18),whiteSpace:"nowrap"},children:[e&&(0,i.jsx)(e.type,{className:s.join(" "),...e.props,style:{fontSize:o||18,verticalAlign:"bottom"}}),n&&(0,i.jsx)("span",{style:{marginLeft:4,verticalAlign:"bottom"},children:n})]})};function c(){const{innerWidth:e,innerHeight:t}=window;return{width:e,height:t}}const u=()=>{const[e,t]=(0,n.useState)(c());return(0,n.useEffect)((()=>{function e(){t(c())}return window.addEventListener("resize",e),()=>window.removeEventListener("resize",e)}),[]),e},f=({width:e,heights:t,children:r})=>{const n=t.reduce(((e,t)=>e+t),0),o=l().Children.toArray(r).map((e=>e));return(0,i.jsx)("div",{className:"VBoxLayout",style:{position:"relative",width:e,height:n},children:o.map(((r,n)=>r?(0,i.jsx)("div",{style:{position:"absolute",overflow:"hidden",background:"white",left:0,top:t.slice(0,n).reduce(((e,t)=>e+t),0),width:e,height:t[n]},children:(0,i.jsx)(r.type,{...r.props,width:e,height:t[n]})},n):(0,i.jsx)("span",{})))})},d=e=>null!==e&&"string"==typeof e,p=e=>null!==e&&"function"==typeof e,h=e=>null!==e&&"number"==typeof e,y=e=>null===e,b=e=>null!==e&&"boolean"==typeof e,w=e=>t=>{for(let r of e)if(r(t))return!0;return!1},j=e=>{if(p(e)){const t=e;return e=>void 0===e||t(e)}return t=>void 0===t||_(t,e)},k=e=>t=>t===e,g=e=>t=>{if(null!==t&&Array.isArray(t)){for(let r of t)if(!e(r))return!1;return!0}return!1},v=e=>null!==e&&"object"==typeof e,m=(e,t)=>r=>{if(v(r)){for(let i in r)if(!e(i)||!t(r[i]))return!1;return!0}return!1},x=e=>!!v(e)&&S(e),O=e=>S(e),S=e=>{if("string"==typeof e||"number"==typeof e)return!0;if(!v(e))return!1;const t=e=>null===e||typeof e>"u"||"string"==typeof e||"boolean"==typeof e||"number"==typeof e||Array.isArray(e)||(e=>"[object Object]"===Object.prototype.toString.call(e))(e);if(!t(e))return!1;for(let r in e)if(e.hasOwnProperty(r)&&(!t(e[r])||null!==e[r]&&"object"==typeof e[r]&&!S(e[r])))return!1;return!0},_=(e,t,r)=>{const i=r||{};if(!e)return i.callback&&i.callback("x is undefined/null."),!1;if("object"!=typeof e)return i.callback&&i.callback("x is not an Object."),!1;for(let r in e)if(!(r in t)&&!i.allowAdditionalFields)return i.callback&&i.callback(`Key not in spec: ${r}`),!1;for(let r in t){const n=t[r];if(p(n)){if(!n(e[r]))return i.callback&&i.callback(`Problem validating: ${r} (${e[r]})`),!1}else{if(!(r in e))return i.callback&&i.callback(`Key not in x: ${r}`),!1;if(!_(e[r],n,{callback:i.callback}))return i.callback&&i.callback(`Value of key > ${r} < itself failed validation.`),!1}}return!0},N=({width:e,height:t,children:r,disableScroll:l})=>{const o=(0,n.useMemo)((()=>({width:e,height:t,position:"relative",overflowY:l?"hidden":"auto"})),[e,t,l]);return(0,i.jsx)("div",{style:o,children:r})}},21020:(e,t,r)=>{var i=r(93345),n=Symbol.for("react.element"),l=Symbol.for("react.fragment"),o=Object.prototype.hasOwnProperty,s=i.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,a={key:!0,ref:!0,__self:!0,__source:!0};function c(e,t,r){var i,l={},c=null,u=null;for(i in void 0!==r&&(c=""+r),void 0!==t.key&&(c=""+t.key),void 0!==t.ref&&(u=t.ref),t)o.call(t,i)&&!a.hasOwnProperty(i)&&(l[i]=t[i]);if(e&&e.defaultProps)for(i in t=e.defaultProps)void 0===l[i]&&(l[i]=t[i]);return{$$typeof:n,type:e,key:c,ref:u,props:l,_owner:s.current}}t.Fragment=l,t.jsx=c,t.jsxs=c},74848:(e,t,r)=>{e.exports=r(21020)}}]);