"use strict";(self.webpackChunkneurosift_jp=self.webpackChunkneurosift_jp||[]).push([[969],{84980:(n,t,e)=>{e.d(t,{C:()=>r});const r=function(n){if(null==n)return i;if("string"==typeof n)return t=n,o((function(n){return n&&n.type===t}));var t;if("object"==typeof n)return Array.isArray(n)?function(n){const t=[];let e=-1;for(;++e<n.length;)t[e]=r(n[e]);return o((function(...n){let e=-1;for(;++e<t.length;)if(t[e].call(this,...n))return!0;return!1}))}(n):function(n){return o((function(t){let e;for(e in n)if(t[e]!==n[e])return!1;return!0}))}(n);if("function"==typeof n)return o(n);throw new Error("Expected function, string, or object as test")};function o(n){return function(t,...e){return Boolean(t&&"object"==typeof t&&"type"in t&&Boolean(n.call(this,t,...e)))}}function i(){return!0}},68969:(n,t,e)=>{e.d(t,{_Z:()=>i,VG:()=>u});var r=e(84980);const o=!1,i="skip",u=function(n,t,e,u){"function"==typeof t&&"function"!=typeof e&&(u=e,e=t,t=null);const f=(0,r.C)(t),c=u?-1:1;!function n(r,l,s){const p=r&&"object"==typeof r?r:{};if("string"==typeof p.type){const n="string"==typeof p.tagName?p.tagName:"string"==typeof p.name?p.name:void 0;Object.defineProperty(a,"name",{value:"node ("+r.type+(n?"<"+n+">":"")+")"})}return a;function a(){let p,a,y,h=[];if((!t||f(r,l,s[s.length-1]||null))&&(h=function(n){return Array.isArray(n)?n:"number"==typeof n?[true,n]:[n]}(e(r,s)),h[0]===o))return h;if(r.children&&h[0]!==i)for(a=(u?r.children.length:-1)+c,y=s.concat(r);a>-1&&a<r.children.length;){if(p=n(r.children[a],a,y)(),p[0]===o)return p;a="number"==typeof p[1]?p[1]:a+c}return h}}(n,void 0,[])()}}}]);