"use strict";(self.webpackChunkneurosift_jp=self.webpackChunkneurosift_jp||[]).push([[45,47,377],{55285:(e,r,n)=>{function t(){return t=Object.assign?Object.assign.bind():function(e){for(var r=1;r<arguments.length;r++){var n=arguments[r];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},t.apply(null,arguments)}n.d(r,{A:()=>t})},36289:(e,r,n)=>{function t(e){var r=Object.create(null);return function(n){return void 0===r[n]&&(r[n]=e(n)),r[n]}}n.d(r,{A:()=>t})},54615:(e,r,n)=>{n.d(r,{J:()=>h});var t={animationIterationCount:1,aspectRatio:1,borderImageOutset:1,borderImageSlice:1,borderImageWidth:1,boxFlex:1,boxFlexGroup:1,boxOrdinalGroup:1,columnCount:1,columns:1,flex:1,flexGrow:1,flexPositive:1,flexShrink:1,flexNegative:1,flexOrder:1,gridRow:1,gridRowEnd:1,gridRowSpan:1,gridRowStart:1,gridColumn:1,gridColumnEnd:1,gridColumnSpan:1,gridColumnStart:1,msGridRow:1,msGridRowSpan:1,msGridColumn:1,msGridColumnSpan:1,fontWeight:1,lineHeight:1,opacity:1,order:1,orphans:1,scale:1,tabSize:1,widows:1,zIndex:1,zoom:1,WebkitLineClamp:1,fillOpacity:1,floodOpacity:1,stopOpacity:1,strokeDasharray:1,strokeDashoffset:1,strokeMiterlimit:1,strokeOpacity:1,strokeWidth:1},o=n(36289),i=!1,a=/[A-Z]|^ms/g,s=/_EMO_([^_]+?)_([^]*?)_EMO_/g,l=function(e){return 45===e.charCodeAt(1)},u=function(e){return null!=e&&"boolean"!=typeof e},c=(0,o.A)((function(e){return l(e)?e:e.replace(a,"-$&").toLowerCase()})),f=function(e,r){switch(e){case"animation":case"animationName":if("string"==typeof r)return r.replace(s,(function(e,r,n){return m={name:r,styles:n,next:m},r}))}return 1===t[e]||l(e)||"number"!=typeof r||0===r?r:r+"px"},d="Component selectors can only be used in conjunction with @emotion/babel-plugin, the swc Emotion plugin, or another Emotion-aware compiler transform.";function v(e,r,n){if(null==n)return"";var t=n;if(void 0!==t.__emotion_styles)return t;switch(typeof n){case"boolean":return"";case"object":var o=n;if(1===o.anim)return m={name:o.name,styles:o.styles,next:m},o.name;var a=n;if(void 0!==a.styles){var s=a.next;if(void 0!==s)for(;void 0!==s;)m={name:s.name,styles:s.styles,next:m},s=s.next;return a.styles+";"}return function(e,r,n){var t="";if(Array.isArray(n))for(var o=0;o<n.length;o++)t+=v(e,r,n[o])+";";else for(var a in n){var s=n[a];if("object"!=typeof s){var l=s;null!=r&&void 0!==r[l]?t+=a+"{"+r[l]+"}":u(l)&&(t+=c(a)+":"+f(a,l)+";")}else{if("NO_COMPONENT_SELECTOR"===a&&i)throw new Error(d);if(!Array.isArray(s)||"string"!=typeof s[0]||null!=r&&void 0!==r[s[0]]){var m=v(e,r,s);switch(a){case"animation":case"animationName":t+=c(a)+":"+m+";";break;default:t+=a+"{"+m+"}"}}else for(var p=0;p<s.length;p++)u(s[p])&&(t+=c(a)+":"+f(a,s[p])+";")}}return t}(e,r,n);case"function":if(void 0!==e){var l=m,p=n(e);return m=l,v(e,r,p)}}var h=n;if(null==r)return h;var y=r[h];return void 0!==y?y:h}var m,p=/label:\s*([^\s;{]+)\s*(;|$)/g;function h(e,r,n){if(1===e.length&&"object"==typeof e[0]&&null!==e[0]&&void 0!==e[0].styles)return e[0];var t=!0,o="";m=void 0;var i=e[0];null==i||void 0===i.raw?(t=!1,o+=v(n,r,i)):o+=i[0];for(var a=1;a<e.length;a++)o+=v(n,r,e[a]),t&&(o+=i[a]);p.lastIndex=0;for(var s,l="";null!==(s=p.exec(o));)l+="-"+s[1];var u=function(e){for(var r,n=0,t=0,o=e.length;o>=4;++t,o-=4)r=1540483477*(65535&(r=255&e.charCodeAt(t)|(255&e.charCodeAt(++t))<<8|(255&e.charCodeAt(++t))<<16|(255&e.charCodeAt(++t))<<24))+(59797*(r>>>16)<<16),n=1540483477*(65535&(r^=r>>>24))+(59797*(r>>>16)<<16)^1540483477*(65535&n)+(59797*(n>>>16)<<16);switch(o){case 3:n^=(255&e.charCodeAt(t+2))<<16;case 2:n^=(255&e.charCodeAt(t+1))<<8;case 1:n=1540483477*(65535&(n^=255&e.charCodeAt(t)))+(59797*(n>>>16)<<16)}return(((n=1540483477*(65535&(n^=n>>>13))+(59797*(n>>>16)<<16))^n>>>15)>>>0).toString(36)}(o)+l;return{name:u,styles:o,next:m}}},71287:(e,r,n)=>{n.d(r,{i:()=>a,s:()=>i});var t=n(93345),o=!!t.useInsertionEffect&&t.useInsertionEffect,i=o||function(e){return e()},a=o||t.useLayoutEffect},30041:(e,r,n)=>{function t(e,r,n){var t="";return n.split(" ").forEach((function(n){void 0!==e[n]?r.push(e[n]+";"):n&&(t+=n+" ")})),t}n.d(r,{Rk:()=>t,SF:()=>o,sk:()=>i});var o=function(e,r,n){var t=e.key+"-"+r.name;!1===n&&void 0===e.registered[t]&&(e.registered[t]=r.styles)},i=function(e,r,n){o(e,r,n);var t=e.key+"-"+r.name;if(void 0===e.inserted[r.name]){var i=r;do{e.insert(r===i?"."+t:"",i,e.sheet,!0),i=i.next}while(void 0!==i)}}}}]);