/*! For license information please see 738.4f44bdca32b61181c5dd.js.LICENSE.txt */
"use strict";(self.webpackChunkneurosift_jp=self.webpackChunkneurosift_jp||[]).push([[363,738],{22799:(e,r)=>{var t,o=Symbol.for("react.element"),n=Symbol.for("react.portal"),f=Symbol.for("react.fragment"),s=Symbol.for("react.strict_mode"),c=Symbol.for("react.profiler"),u=Symbol.for("react.provider"),i=Symbol.for("react.context"),a=Symbol.for("react.server_context"),l=Symbol.for("react.forward_ref"),y=Symbol.for("react.suspense"),p=Symbol.for("react.suspense_list"),m=Symbol.for("react.memo"),S=Symbol.for("react.lazy"),d=Symbol.for("react.offscreen");function b(e){if("object"==typeof e&&null!==e){var r=e.$$typeof;switch(r){case o:switch(e=e.type){case f:case c:case s:case y:case p:return e;default:switch(e=e&&e.$$typeof){case a:case i:case l:case S:case m:case u:return e;default:return r}}case n:return r}}}t=Symbol.for("react.module.reference"),r.ContextConsumer=i,r.ContextProvider=u,r.Element=o,r.ForwardRef=l,r.Fragment=f,r.Lazy=S,r.Memo=m,r.Portal=n,r.Profiler=c,r.StrictMode=s,r.Suspense=y,r.SuspenseList=p,r.isAsyncMode=function(){return!1},r.isConcurrentMode=function(){return!1},r.isContextConsumer=function(e){return b(e)===i},r.isContextProvider=function(e){return b(e)===u},r.isElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===o},r.isForwardRef=function(e){return b(e)===l},r.isFragment=function(e){return b(e)===f},r.isLazy=function(e){return b(e)===S},r.isMemo=function(e){return b(e)===m},r.isPortal=function(e){return b(e)===n},r.isProfiler=function(e){return b(e)===c},r.isStrictMode=function(e){return b(e)===s},r.isSuspense=function(e){return b(e)===y},r.isSuspenseList=function(e){return b(e)===p},r.isValidElementType=function(e){return"string"==typeof e||"function"==typeof e||e===f||e===c||e===s||e===y||e===p||e===d||"object"==typeof e&&null!==e&&(e.$$typeof===S||e.$$typeof===m||e.$$typeof===u||e.$$typeof===i||e.$$typeof===l||e.$$typeof===t||void 0!==e.getModuleId)},r.typeOf=b},44363:(e,r,t)=>{e.exports=t(22799)},21020:(e,r,t)=>{var o=t(93345),n=Symbol.for("react.element"),f=Symbol.for("react.fragment"),s=Object.prototype.hasOwnProperty,c=o.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,u={key:!0,ref:!0,__self:!0,__source:!0};function i(e,r,t){var o,f={},i=null,a=null;for(o in void 0!==t&&(i=""+t),void 0!==r.key&&(i=""+r.key),void 0!==r.ref&&(a=r.ref),r)s.call(r,o)&&!u.hasOwnProperty(o)&&(f[o]=r[o]);if(e&&e.defaultProps)for(o in r=e.defaultProps)void 0===f[o]&&(f[o]=r[o]);return{$$typeof:n,type:e,key:i,ref:a,props:f,_owner:c.current}}r.Fragment=f,r.jsx=i,r.jsxs=i},74848:(e,r,t)=>{e.exports=t(21020)}}]);