/*! For license information please see 825.8ea86ca41494b4acb503.js.LICENSE.txt */
"use strict";(self.webpackChunkneurosift_jp=self.webpackChunkneurosift_jp||[]).push([[206,825],{2206:(e,t,o)=>{o.r(t),o.d(t,{default:()=>d,useModalWindow:()=>u});var n=o(74848),r=o(90481),i=o(1191),s=o(93345);function l(){const{innerWidth:e,innerHeight:t}=window;return{width:e,height:t}}const d=({onClose:e,visible:t,children:o,padding:d,overflow:u})=>{const{width:a,height:h}=(()=>{const[e,t]=(0,s.useState)(l());return(0,s.useEffect)((()=>{function e(){t(l())}return window.addEventListener("resize",e),()=>window.removeEventListener("resize",e)}),[]),e})(),c=Math.min(a,h),f=c<400?0:c<500?10:c<600?20:c<800?30:c<1e3?40:50,p=void 0!==d?d:20,w=o,v=e?50:0,_=a-2*f-10,b=h-2*f-10,y=b-v;return(0,n.jsx)(i.Dialog,{fullScreen:!0,open:t,onClose:e,style:{zIndex:9999,left:f,top:f,right:f,bottom:f,background:"white",position:"absolute",border:"2px solid #000"},children:(0,n.jsxs)("div",{style:{zIndex:9999,position:"absolute",width:_,height:b,overflow:"hidden"},children:[(0,n.jsx)("div",{style:{position:"absolute",padding:20},children:e&&(0,n.jsx)(i.IconButton,{onClick:e,children:(0,n.jsx)(r.Close,{})})}),(0,n.jsx)("div",{style:{position:"absolute",left:p,width:_-2*p,top:v+p,height:y-2*p,overflow:u||"auto"},children:w&&(0,n.jsx)(w.type,{...w.props,width:_-2*p,height:y-2*p})})]})})},u=()=>{const[e,t]=(0,s.useState)(!1),o=(0,s.useCallback)((()=>{t(!0)}),[]),n=(0,s.useCallback)((()=>{t(!1)}),[]);return(0,s.useMemo)((()=>({visible:e,handleOpen:o,handleClose:n})),[e,o,n])}},21020:(e,t,o)=>{var n=o(93345),r=Symbol.for("react.element"),i=Symbol.for("react.fragment"),s=Object.prototype.hasOwnProperty,l=n.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,d={key:!0,ref:!0,__self:!0,__source:!0};function u(e,t,o){var n,i={},u=null,a=null;for(n in void 0!==o&&(u=""+o),void 0!==t.key&&(u=""+t.key),void 0!==t.ref&&(a=t.ref),t)s.call(t,n)&&!d.hasOwnProperty(n)&&(i[n]=t[n]);if(e&&e.defaultProps)for(n in t=e.defaultProps)void 0===i[n]&&(i[n]=t[n]);return{$$typeof:r,type:e,key:u,ref:a,props:i,_owner:l.current}}t.Fragment=i,t.jsx=u,t.jsxs=u},74848:(e,t,o)=>{e.exports=o(21020)}}]);