/*! For license information please see 363.0b94082137b03f3afb2c.js.LICENSE.txt */
"use strict";(self.webpackChunkneurosift_jp=self.webpackChunkneurosift_jp||[]).push([[363],{22799:(e,t)=>{var r,o=Symbol.for("react.element"),n=Symbol.for("react.portal"),f=Symbol.for("react.fragment"),c=Symbol.for("react.strict_mode"),s=Symbol.for("react.profiler"),u=Symbol.for("react.provider"),i=Symbol.for("react.context"),a=Symbol.for("react.server_context"),l=Symbol.for("react.forward_ref"),y=Symbol.for("react.suspense"),p=Symbol.for("react.suspense_list"),m=Symbol.for("react.memo"),S=Symbol.for("react.lazy"),b=Symbol.for("react.offscreen");function d(e){if("object"==typeof e&&null!==e){var t=e.$$typeof;switch(t){case o:switch(e=e.type){case f:case s:case c:case y:case p:return e;default:switch(e=e&&e.$$typeof){case a:case i:case l:case S:case m:case u:return e;default:return t}}case n:return t}}}r=Symbol.for("react.module.reference"),t.ContextConsumer=i,t.ContextProvider=u,t.Element=o,t.ForwardRef=l,t.Fragment=f,t.Lazy=S,t.Memo=m,t.Portal=n,t.Profiler=s,t.StrictMode=c,t.Suspense=y,t.SuspenseList=p,t.isAsyncMode=function(){return!1},t.isConcurrentMode=function(){return!1},t.isContextConsumer=function(e){return d(e)===i},t.isContextProvider=function(e){return d(e)===u},t.isElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===o},t.isForwardRef=function(e){return d(e)===l},t.isFragment=function(e){return d(e)===f},t.isLazy=function(e){return d(e)===S},t.isMemo=function(e){return d(e)===m},t.isPortal=function(e){return d(e)===n},t.isProfiler=function(e){return d(e)===s},t.isStrictMode=function(e){return d(e)===c},t.isSuspense=function(e){return d(e)===y},t.isSuspenseList=function(e){return d(e)===p},t.isValidElementType=function(e){return"string"==typeof e||"function"==typeof e||e===f||e===s||e===c||e===y||e===p||e===b||"object"==typeof e&&null!==e&&(e.$$typeof===S||e.$$typeof===m||e.$$typeof===u||e.$$typeof===i||e.$$typeof===l||e.$$typeof===r||void 0!==e.getModuleId)},t.typeOf=d},44363:(e,t,r)=>{e.exports=r(22799)}}]);